/* tslint:disable */
/**
* Resolution of the board.
*/
export enum Resolution {
  Low,
  Middle,
  High,
}
/**
* Represents the kind of cell. `preset()` method returns a preset cell constants for
* the kind.
*/
export enum LandKind {
  Sea,
  Mountain,
  Forest,
  Plain,
  Town,
  Top,
  Highland,
  DeepSea,
  Path,
}
/**
* Represents one board generated by random map generator.
*/
export class Board {
  free(): void;
/**
* Returns width as number of cells in number
* @returns {number} 
*/
  width(): number;
/**
* Returns height as number of cells in number
* @returns {number} 
*/
  height(): number;
/**
* Returns cell at position (x, y).
* @param {number} x 
* @param {number} y 
* @returns {Cell} 
*/
  at(x: number, y: number): Cell;
/**
* Returns a board by serializing as JSON. On failure, it returns `None` (`undefined` in
* JavaScript)
* @returns {string} 
*/
  as_json(): string;
}
/**
* Represents one cell in board. In contrast to `land::Land`, it only contains its land kind and
* altitude in order to reduce total memory size.
*/
export class Cell {
  free(): void;
  kind: number;
  altitude: number;
/**
* Returns color code of land as `#rrggbb` format string from land kind.
* When no color is set, returns `None`. It means `undefined` in JavaScript side.
* @returns {string} 
*/
  color_code(): string;
/**
* Returns RGB color as u32. 0~8 bits for B, 9~16 bits for G, 17~24 bits for R.
* When no color is set, returns `None`. It means `undefined` in JavaScript side.
* @returns {number | undefined} 
*/
  rgb_color(): number | undefined;
/**
* Returns legend of land as string from land kind.
* @returns {string} 
*/
  legend(): string;
}
/**
* Represents random map generator. In contrast to `gen::RandomBoardGen`, it only provides limited
* functionality. It cannot be initialized with specific seed. And map resolution is always detected
* from its size.
*/
export class Generator {
  free(): void;
/**
* Create a new generator instance initialized with thread-default random number generator.
* No seedable RNG is provided for Wasm interface for now.
* @returns {Generator} 
*/
  static new(): Generator;
/**
* Generates random map board with given width and height. Parameters are in number of cells.
* @param {number} width 
* @param {number} height 
* @returns {Board} 
*/
  gen_auto(width: number, height: number): Board;
/**
* Generates random map board with given resolution, width and height. Width and height are
* in number of cells.
* @param {number} res 
* @param {number} width 
* @param {number} height 
* @returns {Board} 
*/
  gen(res: number, width: number, height: number): Board;
}
