import * as wasm from './world_map_gen_bg';

/**
* Resolution of the board.
*/
export const Resolution = Object.freeze({ Low:0,Middle:1,High:2, });

let cachedTextDecoder = new TextDecoder('utf-8');

let cachegetUint8Memory = null;
function getUint8Memory() {
    if (cachegetUint8Memory === null || cachegetUint8Memory.buffer !== wasm.memory.buffer) {
        cachegetUint8Memory = new Uint8Array(wasm.memory.buffer);
    }
    return cachegetUint8Memory;
}

function getStringFromWasm(ptr, len) {
    return cachedTextDecoder.decode(getUint8Memory().subarray(ptr, ptr + len));
}

let cachedGlobalArgumentPtr = null;
function globalArgumentPtr() {
    if (cachedGlobalArgumentPtr === null) {
        cachedGlobalArgumentPtr = wasm.__wbindgen_global_argument_ptr();
    }
    return cachedGlobalArgumentPtr;
}

let cachegetUint32Memory = null;
function getUint32Memory() {
    if (cachegetUint32Memory === null || cachegetUint32Memory.buffer !== wasm.memory.buffer) {
        cachegetUint32Memory = new Uint32Array(wasm.memory.buffer);
    }
    return cachegetUint32Memory;
}
/**
* Represents the kind of cell. `preset()` method returns a preset cell constants for
* the kind.
*/
export const LandKind = Object.freeze({ Sea:0,Mountain:1,Forest:2,Plain:3,Town:4,Top:5,Highland:6,DeepSea:7,Path:8, });

const heap = new Array(32);

heap.fill(undefined);

heap.push(undefined, null, true, false);

let heap_next = heap.length;

function addHeapObject(obj) {
    if (heap_next === heap.length) heap.push(heap.length + 1);
    const idx = heap_next;
    heap_next = heap[idx];

    heap[idx] = obj;
    return idx;
}

export function __wbg_new_3a746f2619705add(arg0, arg1) {
    let varg0 = getStringFromWasm(arg0, arg1);
    return addHeapObject(new Function(varg0));
}

function getObject(idx) { return heap[idx]; }

export function __wbg_call_f54d3a6dadb199ca(arg0, arg1) {
    return addHeapObject(getObject(arg0).call(getObject(arg1)));
}

export function __wbg_self_ac379e780a0d8b94(arg0) {
    return addHeapObject(getObject(arg0).self);
}

export function __wbg_crypto_1e4302b85d4f64a2(arg0) {
    return addHeapObject(getObject(arg0).crypto);
}

export function __wbg_getRandomValues_1b4ba144162a5c9e(arg0) {
    return addHeapObject(getObject(arg0).getRandomValues);
}

function getArrayU8FromWasm(ptr, len) {
    return getUint8Memory().subarray(ptr / 1, ptr / 1 + len);
}

export function __wbg_getRandomValues_1ef11e888e5228e9(arg0, arg1, arg2) {
    let varg1 = getArrayU8FromWasm(arg1, arg2);
    getObject(arg0).getRandomValues(varg1);
}

export function __wbg_require_6461b1e9a0d7c34a(arg0, arg1) {
    let varg0 = getStringFromWasm(arg0, arg1);
    return addHeapObject(require(varg0));
}

export function __wbg_randomFillSync_1b52c8482374c55b(arg0, arg1, arg2) {
    let varg1 = getArrayU8FromWasm(arg1, arg2);
    getObject(arg0).randomFillSync(varg1);
}

export function __wbindgen_is_undefined(i) { return getObject(i) === undefined ? 1 : 0; }

export function __wbindgen_jsval_eq(a, b) { return getObject(a) === getObject(b) ? 1 : 0; }

export function __wbindgen_throw(ptr, len) {
    throw new Error(getStringFromWasm(ptr, len));
}

function freeBoard(ptr) {

    wasm.__wbg_board_free(ptr);
}
/**
* Represents one board generated by random map generator.
*/
export class Board {

    static __wrap(ptr) {
        const obj = Object.create(Board.prototype);
        obj.ptr = ptr;

        return obj;
    }

    free() {
        const ptr = this.ptr;
        this.ptr = 0;
        freeBoard(ptr);
    }

    /**
    * Returns width as number of cells in number
    * @returns {number}
    */
    width() {
        return wasm.board_width(this.ptr) >>> 0;
    }
    /**
    * Returns height as number of cells in number
    * @returns {number}
    */
    height() {
        return wasm.board_height(this.ptr) >>> 0;
    }
    /**
    * Returns cell at position (x, y).
    * @param {number} x
    * @param {number} y
    * @returns {Cell}
    */
    at(x, y) {
        return Cell.__wrap(wasm.board_at(this.ptr, x, y));
    }
    /**
    * Returns a board by serializing as JSON. On failure, it returns `None` (`undefined` in
    * JavaScript)
    * @returns {string}
    */
    as_json() {
        const retptr = globalArgumentPtr();
        wasm.board_as_json(retptr, this.ptr);
        const mem = getUint32Memory();
        const rustptr = mem[retptr / 4];
        const rustlen = mem[retptr / 4 + 1];
        if (rustptr === 0) return;
        const realRet = getStringFromWasm(rustptr, rustlen).slice();
        wasm.__wbindgen_free(rustptr, rustlen * 1);
        return realRet;

    }
}

function freeCell(ptr) {

    wasm.__wbg_cell_free(ptr);
}
/**
* Represents one cell in board. In contrast to `land::Land`, it only contains its land kind and
* altitude in order to reduce total memory size.
*/
export class Cell {

    static __wrap(ptr) {
        const obj = Object.create(Cell.prototype);
        obj.ptr = ptr;

        return obj;
    }

    free() {
        const ptr = this.ptr;
        this.ptr = 0;
        freeCell(ptr);
    }

    /**
    * Kind for the cell (Sea, Forest, Mountain, ...)
    * @returns {number}
    */
    get kind() {
        return wasm.__wbg_get_cell_kind(this.ptr);
    }
    set kind(arg0) {
        return wasm.__wbg_set_cell_kind(this.ptr, arg0);
    }
    /**
    * Altitude of the cell in 0..99
    * @returns {number}
    */
    get altitude() {
        return wasm.__wbg_get_cell_altitude(this.ptr);
    }
    set altitude(arg0) {
        return wasm.__wbg_set_cell_altitude(this.ptr, arg0);
    }
    /**
    * Returns color code of land as `#rrggbb` format string from land kind.
    * When no color is set, returns `None`. It means `undefined` in JavaScript side.
    * @returns {string}
    */
    color_code() {
        const retptr = globalArgumentPtr();
        wasm.cell_color_code(retptr, this.ptr);
        const mem = getUint32Memory();
        const rustptr = mem[retptr / 4];
        const rustlen = mem[retptr / 4 + 1];
        if (rustptr === 0) return;
        const realRet = getStringFromWasm(rustptr, rustlen).slice();
        wasm.__wbindgen_free(rustptr, rustlen * 1);
        return realRet;

    }
    /**
    * Returns RGB color as u32. 0~8 bits for B, 9~16 bits for G, 17~24 bits for R.
    * When no color is set, returns `None`. It means `undefined` in JavaScript side.
    * @returns {number | undefined}
    */
    rgb_color() {
        const retptr = globalArgumentPtr();

        wasm.cell_rgb_color(retptr, this.ptr);
        const present = getUint32Memory()[retptr / 4];
        const value = getUint32Memory()[retptr / 4 + 1];
        return present === 0 ? undefined : value;

    }
    /**
    * Returns legend of land as string from land kind.
    * @returns {string}
    */
    legend() {
        const retptr = globalArgumentPtr();
        wasm.cell_legend(retptr, this.ptr);
        const mem = getUint32Memory();
        const rustptr = mem[retptr / 4];
        const rustlen = mem[retptr / 4 + 1];

        const realRet = getStringFromWasm(rustptr, rustlen).slice();
        wasm.__wbindgen_free(rustptr, rustlen * 1);
        return realRet;

    }
}

function freeGenerator(ptr) {

    wasm.__wbg_generator_free(ptr);
}
/**
* Represents random map generator. In contrast to `gen::RandomBoardGen`, it only provides limited
* functionality. It cannot be initialized with specific seed. And map resolution is always detected
* from its size.
*/
export class Generator {

    static __wrap(ptr) {
        const obj = Object.create(Generator.prototype);
        obj.ptr = ptr;

        return obj;
    }

    free() {
        const ptr = this.ptr;
        this.ptr = 0;
        freeGenerator(ptr);
    }

    /**
    * Create a new generator instance initialized with thread-default random number generator.
    * No seedable RNG is provided for Wasm interface for now.
    * @returns {Generator}
    */
    static new() {
        return Generator.__wrap(wasm.generator_new());
    }
    /**
    * Generates random map board with given width and height. Parameters are in number of cells.
    * @param {number} width
    * @param {number} height
    * @returns {Board}
    */
    gen_auto(width, height) {
        return Board.__wrap(wasm.generator_gen_auto(this.ptr, width, height));
    }
    /**
    * Generates random map board with given resolution, width and height. Width and height are
    * in number of cells.
    * @param {number} res
    * @param {number} width
    * @param {number} height
    * @returns {Board}
    */
    gen(res, width, height) {
        return Board.__wrap(wasm.generator_gen(this.ptr, res, width, height));
    }
}

function dropObject(idx) {
    if (idx < 36) return;
    heap[idx] = heap_next;
    heap_next = idx;
}

export function __wbindgen_object_drop_ref(i) { dropObject(i); }

